/*******************************************************************************
 * Copyright 2005  Hon Hai Precision Ind. Co. Ltd.
 * All Rights Reserved.
 * No portions of this material shall be reproduced in any form without the
 * written permission of Hon Hai Precision Ind. Co. Ltd.
 *
 * All information contained in this document is Hon Hai Precision Ind.
 * Co. Ltd. company private, proprietary, and trade secret property and
 * are protected by international intellectual property laws and treaties.
 *
 ******************************************************************************/

/*******************************************************************************
 ***  Filename:
 ***      acl_logd.c
 ***
 ***  Description:
 ***      
 ***      A task to monitor wireless association list if ACL is enabled.
 ***
 ***  History:
 ***
 ***   Modify Reason               Author          Date      Search Flag(Option)
 ***----------------------------------------------------------------------------
 ***   Created                     Eric Huang      2007/01/04
 ******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <bcmnvram.h>
#include <arpa/inet.h>
#include <shutils.h>
#include <bcmutils.h>
#include <typedefs.h>
#include <wlioctl.h>
#include <wlutils.h>
#include "fox_wl.h"

#define stricmp strcasecmp
#define strnicmp strncasecmp

#define MAX_STA_COUNT   64 /* maximum ACL list defined in NGR's router spec rev1.0 */
#define LOG_DURATION    30

#define WLC_BAND_A  WLC_BAND_5G
#define WLC_BAND_B  WLC_BAND_2G

/*
 * buffer length needed for wlc_format_ssid
 * 32 SSID chars, max of 4 chars for each SSID char "\xFF", plus NULL.
 */
#define SSID_FMT_BUF_LEN	((4 * DOT11_MAX_SSID_LEN) + 1)

#define isprint(c) bcm_isprint(c)
/*
 * Country names and abbreviations from ISO 3166
 */
typedef struct {
	char *name;	/* Long name */
	char *abbrev;	/* Abbreviation */
} cntry_name_t;

#define MINIOPT_MAXKEY	128
typedef struct miniopt {
	char* name;		/* name for prompt in error strings */
	char* flags;		/* set of chars that are option flags
				 * with no args */
	int consumed;		/* number of argv entries cosumed in
				 * the most recent call to miniopt() */
	bool opt_end;		/* at end of options since a "--" flag
				 * was encountered */
	bool positional;	
	bool good_int;		/* 'val' member is the result of a sucessful
				 * strtol conversion of the option value */
	char opt;		
	char key[MINIOPT_MAXKEY];	
	char* valstr;		/* positional param, or value for the option,
				 * or null if the option had
				 * no accompanying value */
	int val;		/* strtol translation of valstr */
} miniopt_t;

/* integer output format, default to signed integer */
static uint8 int_fmt = INT_FMT_DEC;
static int wl_maclist(char *wlif, cmd_t *cmd, char **argv);
static int wl_radio(char *wl, cmd_t *cmd, char **argv);
static int wl_band(char *wl, cmd_t *cmd, char **argv);
static int wl_void(char *wl, cmd_t *cmd, char **argv);
int wl_country(char *wl, cmd_t *cmd, char **argv);
static int wl_ssid(char *wl, cmd_t *cmd, char **argv);
static int wl_channel(char *wl, cmd_t *cmd, char **argv);
static int wl_rate_mrate(char *wl, cmd_t *cmd, char **argv);
static cntry_name_t *wlc_cntry_abbrev_to_country(char *abbrev);
static cntry_name_t *wlc_cntry_name_to_country(char *long_name);
static int wl_cfg_option(char **argv, char *fn_name, int *bsscfg_idx, int *consumed);
int wl_format_ssid(char* buf, uint8* ssid, int ssid_len);
static void wl_printrate(int val);
static int rate_string2int(char *s);
void miniopt_init(miniopt_t *t, char* name, char* flags);
int miniopt(miniopt_t *t, char **argv);
static char *rate_int2string(char *buf, int val);
static int wl_rssi(char *wl, cmd_t *cmd, char **argv);/* Added by zacker,01/26/2008 */

//cmd_t wl_varcmd = { "var", wl_varint, -1, -1, "unrecognized name, type -h for help" } ;
char *av0 = "";

const unsigned char bcm_ctype[] = {
    _BCM_C,_BCM_C,_BCM_C,_BCM_C,_BCM_C,_BCM_C,_BCM_C,_BCM_C,			/* 0-7 */
    _BCM_C, _BCM_C|_BCM_S, _BCM_C|_BCM_S, _BCM_C|_BCM_S, _BCM_C|_BCM_S, _BCM_C|_BCM_S, _BCM_C,
	_BCM_C,	/* 8-15 */
	_BCM_C,_BCM_C,_BCM_C,_BCM_C,_BCM_C,_BCM_C,_BCM_C,_BCM_C,			/* 16-23 */
	_BCM_C,_BCM_C,_BCM_C,_BCM_C,_BCM_C,_BCM_C,_BCM_C,_BCM_C,			/* 24-31 */
	_BCM_S|_BCM_SP,_BCM_P,_BCM_P,_BCM_P,_BCM_P,_BCM_P,_BCM_P,_BCM_P,		/* 32-39 */
	_BCM_P,_BCM_P,_BCM_P,_BCM_P,_BCM_P,_BCM_P,_BCM_P,_BCM_P,			/* 40-47 */
	_BCM_D,_BCM_D,_BCM_D,_BCM_D,_BCM_D,_BCM_D,_BCM_D,_BCM_D,			/* 48-55 */
	_BCM_D,_BCM_D,_BCM_P,_BCM_P,_BCM_P,_BCM_P,_BCM_P,_BCM_P,			/* 56-63 */
	_BCM_P, _BCM_U|_BCM_X, _BCM_U|_BCM_X, _BCM_U|_BCM_X, _BCM_U|_BCM_X, _BCM_U|_BCM_X,
	_BCM_U|_BCM_X, _BCM_U, /* 64-71 */
	_BCM_U,_BCM_U,_BCM_U,_BCM_U,_BCM_U,_BCM_U,_BCM_U,_BCM_U,			/* 72-79 */
	_BCM_U,_BCM_U,_BCM_U,_BCM_U,_BCM_U,_BCM_U,_BCM_U,_BCM_U,			/* 80-87 */
	_BCM_U,_BCM_U,_BCM_U,_BCM_P,_BCM_P,_BCM_P,_BCM_P,_BCM_P,			/* 88-95 */
	_BCM_P, _BCM_L|_BCM_X, _BCM_L|_BCM_X, _BCM_L|_BCM_X, _BCM_L|_BCM_X, _BCM_L|_BCM_X,
	_BCM_L|_BCM_X, _BCM_L, /* 96-103 */
	_BCM_L,_BCM_L,_BCM_L,_BCM_L,_BCM_L,_BCM_L,_BCM_L,_BCM_L, /* 104-111 */
	_BCM_L,_BCM_L,_BCM_L,_BCM_L,_BCM_L,_BCM_L,_BCM_L,_BCM_L, /* 112-119 */
	_BCM_L,_BCM_L,_BCM_L,_BCM_P,_BCM_P,_BCM_P,_BCM_P,_BCM_C, /* 120-127 */
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,		/* 128-143 */
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,		/* 144-159 */
	_BCM_S|_BCM_SP, _BCM_P, _BCM_P, _BCM_P, _BCM_P, _BCM_P, _BCM_P, _BCM_P, _BCM_P, _BCM_P,
	_BCM_P, _BCM_P, _BCM_P, _BCM_P, _BCM_P, _BCM_P,	/* 160-175 */
	_BCM_P, _BCM_P, _BCM_P, _BCM_P, _BCM_P, _BCM_P, _BCM_P, _BCM_P, _BCM_P, _BCM_P, _BCM_P,
	_BCM_P, _BCM_P, _BCM_P, _BCM_P, _BCM_P,	/* 176-191 */
	_BCM_U, _BCM_U, _BCM_U, _BCM_U, _BCM_U, _BCM_U, _BCM_U, _BCM_U, _BCM_U, _BCM_U, _BCM_U,
	_BCM_U, _BCM_U, _BCM_U, _BCM_U, _BCM_U,	/* 192-207 */
	_BCM_U, _BCM_U, _BCM_U, _BCM_U, _BCM_U, _BCM_U, _BCM_U, _BCM_P, _BCM_U, _BCM_U, _BCM_U,
	_BCM_U, _BCM_U, _BCM_U, _BCM_U, _BCM_L,	/* 208-223 */
	_BCM_L, _BCM_L, _BCM_L, _BCM_L, _BCM_L, _BCM_L, _BCM_L, _BCM_L, _BCM_L, _BCM_L, _BCM_L,
	_BCM_L, _BCM_L, _BCM_L, _BCM_L, _BCM_L,	/* 224-239 */
	_BCM_L, _BCM_L, _BCM_L, _BCM_L, _BCM_L, _BCM_L, _BCM_L, _BCM_P, _BCM_L, _BCM_L, _BCM_L,
	_BCM_L, _BCM_L, _BCM_L, _BCM_L, _BCM_L /* 240-255 */
};

cntry_name_t cntry_names[] = {


{"UNITED STATES",		"US"},

{NULL, 			 NULL}
};

cmd_t wl_cmds[] = {
    { "assoclist", wl_maclist, WLC_GET_ASSOCLIST, -1,
	  "AP only: Get the list of associated MAC addresses."},
	{ "radio", wl_radio, WLC_GET_RADIO, WLC_SET_RADIO,
	  "Set the radio on or off.\n"
	  "\t\"on\" or \"off\"" },
	{ "band", wl_band, WLC_GET_BAND, WLC_SET_BAND,
	  "Returns or sets the current band\n"
	  "\tauto - auto switch between available bands (default)\n"
	  "\ta - force use of 802.11a band\n"
	  "\tb - force use of 802.11b band" },
	{ "rate", wl_rate_mrate, WLC_GET_RATE, -1, 
	  "force a fixed rate:\n"
	  "\tvalid values for 802.11a are (6, 9, 12, 18, 24, 36, 48, 54)\n"
	  "\tvalid values for 802.11b are (1, 2, 5.5, 11)\n"
	  "\tvalid values for 802.11g are (1, 2, 5.5, 6, 9, 11, 12, 18, 24, 36, 48, 54)\n"
	  "\t-1 (default) means automatically determine the best rate" },
	{ "channel", wl_channel, WLC_GET_CHANNEL, WLC_SET_CHANNEL,
	  "Set the channel:\n"
	  "\tvalid channels for 802.11b/g (2.4GHz band) are 1 through 14\n"
	  "\tvalid channels for 802.11a  (5 GHz band) are:\n"
	  "\t\t36, 40, 44, 48, 52, 56, 60, 64,\n"
	  "\t\t100, 104, 108, 112, 116,120, 124, 128, 132, 136, 140,\n"
	  "\t\t149, 153, 157, 161,\n"
	  "\t\t184, 188, 192, 196, 200, 204, 208, 212, 216"},
	{ "ssid", wl_ssid, WLC_GET_SSID, WLC_SET_SSID,
	  "Set or get a configuration's SSID.\n"
	  "\twl ssid [-C num]|[--cfg=num] [<ssid>]\n"
	  "\tIf the configuration index 'num' is not given, configuraion #0 is assumed and\n"
	  "\tsetting will initiate an assoication attempt if in infrastructure mode,\n"
	  "\tor join/creation of an IBSS if in IBSS mode,\n"
	  "\tor creation of a BSS if in AP mode.\n"},
	{ "country", wl_country, WLC_GET_COUNTRY, WLC_SET_COUNTRY,
	  "Select Country code for use with 802.11d\n"
	  "\tUse either long name or abbreviation from ISO 3166.\n"
	  "\tUse 'wl country list [band(a or b)]' for the list of supported countries"},
	{ "up",	wl_void, -1, WLC_UP,
	  "reinitialize and mark adapter up (operational)" },
	{ "down", wl_void, -1, WLC_DOWN,
	  "reset and mark adapter down (disabled)" },
	{ "rssi", wl_rssi, WLC_GET_RSSI, -1,/* Added by zacker,01/26/2008 */
	  "\tReturn the RSSI value of an associated STA with a specified MAC"},
    { NULL, NULL, 0, 0, NULL }};

/* parse/validate the command line arguments */
/*
 * pargv is updated upon return if the first argument is an option.
 * It remains intact otherwise.
 */
int wl_option(char ***pargv, char **pifname, int *phelp)
{
	char *ifname = NULL;
	int help = FALSE;
	int status = CMD_OPT;
	char **argv = *pargv;

	while (*argv) {
		/* select different adapter */
		if (!strcmp(*argv, "-a") || !strcmp(*argv, "-i")) {
			char *opt = *argv++;
			ifname = *argv;
			if (!ifname) {
				fprintf(stderr, "error: expected interface name after option %s\n", opt);
				status = CMD_ERR;
				break;
			}
		}

		/* integer output format */
		else if (!strcmp(*argv, "-d"))
			int_fmt = INT_FMT_DEC;
		else if (!strcmp(*argv, "-u"))
			int_fmt = INT_FMT_UINT;
		else if (!strcmp(*argv, "-x"))
			int_fmt = INT_FMT_HEX;

		/* command usage */
		else if (!strcmp(*argv, "-h"))
			help = TRUE;

		/* start of non wl options */
		else {
			status = CMD_WL;
			break;
		}

		/* consume the argument */
		argv ++;
		break;
	}
	
	*phelp = help;
	*pifname = ifname;
	*pargv = argv;
	
	return status;
}


int main(int argc, char **argv)
{
    //struct ifreq ifr;
    cmd_t *cmd = NULL;
    int err = 0;
    char *ifname = NULL;
    int help = 0;
    int status = CMD_WL;
    
    char *wlif = nvram_safe_get("wl0_ifname");
    av0 = argv[0];

    //memset(&ifr, 0, sizeof(ifr));

    for (++argv; *argv; )
    {
        /* command option */
        if ((status = wl_option(&argv, &ifname, &help)) == CMD_OPT) 
        {
            if (help)
                break;
            //if (ifname)
            //    strncpy(ifr.ifr_name, ifname, IFNAMSIZ);
            continue;
        }
        /* parse error */
        else if (status == CMD_ERR)
            break;
        /* wl command */
        /*
		 * else if (status == CMD_WL)
		 *	;
		 */

        /* use default interface */
        //if (!*ifr.ifr_name)
        //    wl_find(&ifr);
		/* validate the interface */
        //if (!*ifr.ifr_name || wl_check((void *)&ifr)) 
        //{
        //    fprintf(stderr, "%s: wl driver adapter not found\n", av0);
        //    exit(1);
        //}

        /* search for command */
		for (cmd = wl_cmds; cmd->name && strcmp(cmd->name, *argv); cmd++);

		/* defaults to using the set_var and get_var commands */
		//if (cmd->name == NULL)
		//	cmd = &wl_varcmd;

		/* do command */
		if (cmd->name)
			err = (*cmd->func)(wlif, cmd, argv);
		break;
	}

    return err;
    /*
	if (!cmd)
		wl_usage(NULL);
	else if (err == USAGE_ERROR)
		cmd_usage(cmd);
	else if (err == IOCTL_ERROR)
		wl_printlasterror((void *) &ifr);
	return err;
	*/
}


int
wl_get(void *wl, int cmd, void *buf, int len)
{
	return wl_ioctl(wl, cmd, buf, len);
}

int
wl_set(void *wl, int cmd, void *buf, int len)
{
	return wl_ioctl(wl, cmd, buf, len);
}
int wl_ether_atoe(const char *a, unsigned char *n)
{
	char *c = (char *) a;
	int i = 0;

	memset(n, 0, ETHER_ADDR_LEN);
	for (;;) {
		n[i++] = (unsigned char) strtoul(c, &c, 16);
		if (!*c++ || i == ETHER_ADDR_LEN)
			break;
	}
	return (i == ETHER_ADDR_LEN);
}
char *wl_ether_etoa(const unsigned char *n)
{
	static char etoa_buf[ETHER_ADDR_LEN * 3];
	char *c = etoa_buf;
	int i;

	for (i = 0; i < ETHER_ADDR_LEN; i++) {
		if (i)
			*c++ = ':';
		c += sprintf(c, "%02X", n[i] & 0xff);
	}
	return etoa_buf;
}

char buf[WLC_IOCTL_MAXLEN];

static int wl_maclist(char *wlif, cmd_t *cmd, char **argv)
{
    int ret;
	struct maclist *maclist = (struct maclist *) buf;
	struct ether_addr *ea;
	uint i, max = (WLC_IOCTL_MAXLEN - sizeof(int)) / ETHER_ADDR_LEN;
	if (!*++argv) 
	{
        if (cmd->get < 0)
            return -1;
        maclist->count = max;
        if ((ret = wl_get(wlif, cmd->get, maclist, WLC_IOCTL_MAXLEN)) < 0)
            return ret;
        for (i = 0, ea = maclist->ea; i < maclist->count && i < max; i++, ea++)
        {
            printf("%s %s\n", cmd->name, wl_ether_etoa(ea->octet));
        }
        return 0;
    }
    else 
    {
		if (cmd->set < 0)
			return -1;
		/* Clear list */
		maclist->count = 0;
		if (!stricmp(*argv, "none") || !stricmp(*argv, "clear"))
			return wl_set(wlif, cmd->set, maclist, sizeof(int));
		/* Get old list */
		maclist->count = max;
		if ((ret = wl_get(wlif, cmd->get, maclist, WLC_IOCTL_MAXLEN)) < 0)
			return ret;
		/* Append to old list */
		ea = &maclist->ea[maclist->count];
		while (*argv && maclist->count < max) {
			if (!wl_ether_atoe(*argv, ea->octet)){
				printf("Problem parsing MAC address \"%s\".\n", *argv);
				return -1;
			}
			maclist->count++;
			ea++;
			argv++;
		}
		/* Set new list */
		return wl_set(wlif, cmd->set, maclist, WLC_IOCTL_MAXLEN);
	}
    return 0;
}

/* WLC_GET_RADIO and WLC_SET_RADIO in driver operate on radio_disabled which
 * is opposite of "wl radio [1|0]".  So invert for user.
 * In addition, display WL_RADIO_SW_DISABLE and WL_RADIO_HW_DISABLE bits.
 */
static int wl_radio(char *wl, cmd_t *cmd, char **argv)
{
	int ret;
	uint val;

	if (!*++argv) {
		if (cmd->get < 0)
			return -1;			
		if ((ret = wl_get(wl, cmd->get, &val, sizeof(int))) < 0)
			return ret;
		printf("0x%04x\n", val);
		return 0;
	} else {
		if (cmd->set < 0)
			return -1;
		if (!stricmp(*argv, "on"))
			val = WL_RADIO_SW_DISABLE << 16;
		else if (!stricmp(*argv, "off"))
			val = WL_RADIO_SW_DISABLE << 16 | WL_RADIO_SW_DISABLE;
		else
			val = atoi(*argv);
		return wl_set(wl, cmd->set, &val, sizeof(int));
	}
}

static int wl_band(char *wl, cmd_t *cmd, char **argv)
{
	uint band;
	int error;

	error = 0;

	argv++;

	if (*argv == NULL) {	/* get current band */
		if ((error = wl_get(wl, WLC_GET_BAND, &band, sizeof (uint))) < 0)
			return (error);

		if (band == WLC_BAND_AUTO)
			printf("auto\n");
		else if (band == WLC_BAND_A)
			printf("a\n");
		else if (band == WLC_BAND_B)
			printf("b\n");
		else
			printf("unrecognized band value %d\n", band);
	} else {		/* set the band */
		if (!stricmp(*argv, "auto"))
			band = WLC_BAND_AUTO;
		else if (!stricmp(*argv, "a"))
			band = WLC_BAND_A;
		else if (!stricmp(*argv, "b"))
			band = WLC_BAND_B;
		else {
			printf("unsupported band: %s\n", *argv);
			return (-1);
		}

		error = wl_set(wl, WLC_SET_BAND, &band, sizeof (uint));

	}
	return (error);
}

static int wl_void(char *wl, cmd_t *cmd, char **argv)
{
	if (cmd->set < 0)
		return -1;
	return wl_set(wl, cmd->set, NULL, 0);
}

int wl_country(char *wl, cmd_t *cmd, char **argv)
{
	cntry_name_t *cntry;
	char* name = "<unknown>";
	char* abbrev;
	int ret;
	uint i;

	buf[0] = 0;
	if (!*++argv) {
		if ((ret = wl_get(wl, cmd->get, &buf, sizeof (buf))))
			return ret;
		if (strlen(buf) == 0) {
			printf("No country set\n");
			return 0;
		}
		if ((cntry = wlc_cntry_abbrev_to_country(buf)) != NULL){
			name = cntry->name;
		}
		printf("%s/%s\n", buf, name);
		return 0;
	}

	if (!stricmp(*argv, "list")){
		wl_country_list_t *cl = (wl_country_list_t *)buf;
		cntry_name_t *cntry;

		cl->buflen = sizeof(buf);
		cl->count = 0;

		/* band may follow */
		if (*++argv) {
			cl->band_set = TRUE;
			if (!stricmp(*argv, "a"))
				cl->band = WLC_BAND_A;
			else if (!stricmp(*argv, "b") || !stricmp(*argv, "g"))
				cl->band = WLC_BAND_B;
			else {
				printf("unsupported band: %s\n", *argv);
				return -1;
			}
		} else {
			cl->band_set = FALSE;
		}
	
		ret = wl_get(wl, WLC_GET_COUNTRY_LIST, buf, cl->buflen);
		if (ret < 0)
			return ret;
		
		printf("Supported countries: abbreviation and long name\n");
		for (i = 0; i < cl->count; i++) {
			char *abbrev = &cl->country_abbrev[i*WLC_CNTRY_BUF_SZ];
			cntry = wlc_cntry_abbrev_to_country(abbrev);
			printf("%s\t%s\n", abbrev, cntry ? cntry->name : abbrev);
		}
		return ret;
	}

	/* if matching long name, find abbrev name, otherwise assume it's abbrev name */
	if ((cntry = wlc_cntry_name_to_country(*argv)) != NULL)
		abbrev = cntry->abbrev;
	else
		abbrev = *argv;
	
	if (strlen(abbrev) > 3) {
		printf("Parameter \"%s\" was unrecognized as a country name, and is\n"
		       "too long for a max 3 character country abbreviation.\n",
		       abbrev);
		return 1;
	}

	printf("Setting country code using abbreviation: \"%s\"\n", abbrev);
	strcpy(buf, abbrev);

	if (wl_set(wl, cmd->set, buf, strlen(buf) + 1) < 0) {
		printf("Setting country \"%s\" failed !\n", abbrev);
		return 1;
	}

	return(0);
}

/* Set or Get the "ssid" iovar, with an optional config index argument:
 *	wl ssid [-C N]|[--cfg=N] ssid
 * 
 * Option:
 *	-C N
 *	--cfg=N
 *	--config=N
 *	--configuration=N
 *		specify the config index N
 * If cfg index not given on a set, the WLC_SET_SSID ioctl will be used
 */
static int wl_ssid(char *wl, cmd_t *cmd, char **argv)
{

	char ssidbuf[SSID_FMT_BUF_LEN];
	wlc_ssid_t ssid;
	int bsscfg_idx = 0;
	int consumed;
	int error;
	
	argv++;

	/* parse a bsscfg_idx option if present */
	if ((error = wl_cfg_option(argv, "ssid", &bsscfg_idx, &consumed)) != 0)
		return error;

	argv += consumed;

	if (*argv == NULL) {
		if (consumed == 0) {
			/* no config index, use WLC_GET_SSID on the interface */
			error = wl_get(wl, WLC_GET_SSID, &ssid, sizeof(ssid));
		} else {
			/* use "ssid" iovar since a config option was given */
			error = wl_bssiovar_get(wl, "ssid", bsscfg_idx, &ssid, sizeof(ssid));
		}
		if (error < 0)
			return error;

		wl_format_ssid(ssidbuf, ssid.SSID, ssid.SSID_len);
		printf("Current SSID: \"%s\"\n", ssidbuf);
	} else {
		if (strlen(argv[0]) > 32) {
 			fprintf(stderr, "SSID arg \"%s\" must be 32 chars or less\n", argv[0]);
			return -1;
		}
		strcpy(ssid.SSID, argv[0]);
		ssid.SSID_len = strlen(ssid.SSID);

		wl_format_ssid(ssidbuf, ssid.SSID, ssid.SSID_len);
		printf("Setting SSID \"%s\"\n", ssidbuf);

		if (consumed == 0) {
			/* no config index given, use WLC_SET_SSID */
			error = wl_set(wl, WLC_SET_SSID, &ssid, sizeof(wlc_ssid_t));
		} else {
			/* use "ssid" iovar since a config option was given */
			error = wl_bssiovar_set(wl, "ssid", bsscfg_idx, &ssid, sizeof(wlc_ssid_t));
		}
	}
	return error;

}
/* Foxconn added start,zacker,01/26/2008 */
static int wl_rssi(char *wl, cmd_t *cmd, char **argv)
{
	wlc_rssi_t rssi = {0};
	uint8 sta_mac[ETHER_ADDR_LEN] = {0};
	int ret = 0;

	if (!*++argv) {
		printf("A MAC address of station must be specified.\n");
		return -1;
	} else {
			ret = ether_atoe(*argv, sta_mac);
			if (!ret) {
				printf("Invalid MAC address.\n");
				return -1;
			}
			memcpy(&(rssi.ea), sta_mac, sizeof(struct ether_addr));
			rssi.unknown = 0x0041;
			ret = wl_get(wl, cmd->get, &rssi, sizeof(wlc_rssi_t));
			if (ret < 0) {
				rssi.rssi = 0;
			}
			printf("%d\n", rssi.rssi);
			return ret;
	}
}
/* Foxconn added end,zacker,01/26/2008 */

static int wl_channel(char *wl, cmd_t *cmd, char **argv)
{
   
	int ret;
	channel_info_t ci;

	if (!*++argv) {
		memset(&ci, 0, sizeof(ci));
		if ((ret = wl_get(wl, cmd->get, &ci, sizeof(channel_info_t))) < 0)
			return ret;
		if (ci.scan_channel) {
			printf("Scan in progress.\n");
			printf("current scan channel\t%d\n", ci.scan_channel);
		} else {
			printf("No scan in progress.\n");
		}
		printf("current mac channel\t%d\n", ci.hw_channel);
		printf("target channel\t%d\n", ci.target_channel);
		return 0;
	} else {
		ci.target_channel = atoi(*argv);
		ret =  wl_set(wl, cmd->set, &ci.target_channel, sizeof(int));
		return ret;
	}
	
}

/* handles both "rate" and "mrate", which makes the flow a bit complex */
static int wl_rate_mrate(char *wl, cmd_t *cmd, char **argv)
{
  
	int error;
	int val;
	int band;
	int list[3];
	char aname[sizeof("a_mrate") + 1];
	char bgname[sizeof("bg_mrate") + 1];
	char *name;

	sprintf(aname, "a_%s", *argv);
	sprintf(bgname, "bg_%s", *argv);


	if ((error = wl_get(wl, WLC_GET_BAND, &band, sizeof (uint))) < 0)
		return error;

	if ((error = wl_get(wl, WLC_GET_BANDLIST, list, sizeof (list))) < 0)
		return error;

	if (!list[0]) 
		return -1;
	else if(list[0] > 2)
		list[0] = 2;

	if ((!strcmp(*argv, "rate"))) {
		/* it is "rate" */
		if (!*++argv) {
			/* it is "rate" get. handle it here */
			/* WLC_GET_RATE processing */
			if ((error = wl_get(wl, cmd->get, &val, sizeof (int))) < 0)
				return error;
			
			wl_printrate(val);
			return 0;
		}
		else
			--argv; /* move it back for later processing */
	}

	switch (band) {
	case WLC_BAND_AUTO : 
		if(list[0] > 1)
			return -1;
		else if (list[1] == WLC_BAND_A)
			name = (char *)aname;
		else if (list[1] == WLC_BAND_B)
			name = (char *)bgname;
		else
			return -1;			
	
		break;
		
	case WLC_BAND_A : 
		name = (char *)aname;
		break;
		
	case WLC_BAND_B :
		name = (char *)bgname;
		break;
	
	default :
		return -1;
		break;
	}

	if (!*++argv) {
		/* it is "mrate" get */
		if ((error = wl_iovar_getint(wl, name, &val) < 0))
			return error;
		
		if (val == 0)
			printf("auto\n");
		else
			wl_printrate(val);
			
		return 0;			
	} else {
		val = rate_string2int(*argv);
		return wl_iovar_setint(wl, name, val);
	}
		
}

static cntry_name_t *wlc_cntry_abbrev_to_country(char *abbrev)
{
	cntry_name_t *cntry;
	if (!*abbrev || strlen(abbrev) > 3 || strlen(abbrev) < 2)
		return(NULL);
	for (cntry = cntry_names; cntry->name && strnicmp(abbrev, cntry->abbrev, strlen(abbrev));cntry++);
	return (!cntry->name ? NULL : cntry);
}

static cntry_name_t *wlc_cntry_name_to_country(char *long_name)
{
	cntry_name_t *cntry;
	for (cntry = cntry_names; cntry->name && stricmp(long_name, cntry->name);cntry++);
	return (!cntry->name ? NULL : cntry);
}

/* Common routine to check for an option arg specifying the configuration index.
 * Takes the syntax -C num, --cfg=num, --config=num, or --configuration=num
 * Returns -1 if there is a command line parsing error.
 * Returns 0 if no error, and sets *consumed to the number of argv strings
 * used. Sets *bsscfg_idx to the index to use. Will set *bsscfg_idx to zero if there
 * was no config arg.
 */
static int wl_cfg_option(char **argv, char *fn_name, int *bsscfg_idx, int *consumed)
{
	miniopt_t mo;
	int opt_err;
	
	*bsscfg_idx = 0;
	*consumed = 0;

	miniopt_init(&mo, fn_name, NULL);

	/* process the first option */
	opt_err = miniopt(&mo, argv);

	/* check for no args or end of options */
	if (opt_err == -1)
		return 0;
	
	/* check for no options, just a positional arg encountered */
	if (mo.positional)
		return 0;

	/* check for error parsing options */
	if (opt_err == 1)
		return -1;
	
	/* check for -C, --cfg=X, --config=X, --configuration=X */
	if (mo.opt == 'C' ||
	    !strcmp(mo.key, "cfg") ||
	    !strcmp(mo.key, "config") ||
	    !strcmp(mo.key, "configuration")) {
		if (!mo.good_int) {
			fprintf(stderr, "%s: could not parse \"%s\" as an integer for the configuartion index\n",
				fn_name, mo.valstr);
			return -1;
		}
		*bsscfg_idx = mo.val;
		*consumed = mo.consumed;
	}
	
	return 0;
}

int wl_format_ssid(char* buf, uint8* ssid, int ssid_len)
{
	int i, c;
	char *p = buf;

	if (ssid_len > 32) ssid_len = 32;
	
	for (i = 0; i < ssid_len; i++) {
		c = (int)ssid[i];
		if (c == '\\') {
			*p++ = '\\';
			*p++ = '\\';
		} else if (isprint((uchar)c)) {
			*p++ = (char)c;
		} else {
			p += sprintf(p, "\\x%02X", c);
		}
	}
	*p = '\0';
	
	return p - buf;
}

/* take rate arg in units of 500Kbits/s and print it in units of Mbit/s */
static void wl_printrate(int val)
{
	char buf[32];

	printf("%s\n", rate_int2string(buf, val));
}

/* convert rate string in Mbit/s format, like "11", "5.5", to internal 500 Kbit/s units */
static int rate_string2int(char *s)
{
	if (!stricmp(s, "-1"))
		return (0);
	if (!stricmp(s, "5.5"))
		return (11);
	return (atoi(s) * 2);
}

void
miniopt_init(miniopt_t *t, char* name, char* flags)
{
	static char *null_flags = "";
	
	memset(t, 0, sizeof (miniopt_t));
	t->name = name;
	if (flags == NULL)
		t->flags = null_flags;
	else
		t->flags = flags;
}

int 
miniopt(miniopt_t *t, char **argv)
{
	int keylen;
	char *p, *eq, *valstr, *endptr;
	int err = 0;

	t->consumed = 0;
	t->positional = FALSE;
	memset(t->key, 0, MINIOPT_MAXKEY);
	t->opt = '\0';
	t->valstr = NULL;
	t->good_int = FALSE;

	if (*argv == NULL) {
		err = -1;
		goto exit;
	}
	
	p = *argv++;
	t->consumed++;

	if (!t->opt_end && !strcmp(p, "--")) {
		t->opt_end = TRUE;
		if (*argv == NULL) {
			err = -1;
			goto exit;
		}
		p = *argv++;
		t->consumed++;
	}

	if (t->opt_end) {
		t->positional = TRUE;
		valstr = p;
	}
	else if (!strncmp(p, "--", 2)) {
		eq = strchr(p, '=');
		if (eq == NULL) {
			fprintf(stderr, "%s: missing \"=\" in long param \"%s\"\n", t->name, p);
			err = 1;
			goto exit;
		}
		keylen = eq - (p + 2);
		if (keylen > 63) keylen = 63;
		memcpy(t->key, p + 2, keylen);

		valstr = eq + 1;
		if (*valstr == '\0') {
			fprintf(stderr, "%s: missing value after \"=\" in long param \"%s\"\n", t->name , p);
			err = 1;
			goto exit;
		}
	}
	else if (!strncmp(p, "-", 1)) {
		t->opt = p[1];
		if (strlen(p) > 2) {
			fprintf(stderr, "%s: only single char options, error on param \"%s\"\n", t->name, p);
			err = 1;
			goto exit;
		}
		if (strchr(t->flags, t->opt)) {
			/* this is a flag option, no value expected */
			valstr = NULL;
		} else {
			if (*argv == NULL) {
				fprintf(stderr, "%s: missing value parameter after \"%s\"\n", t->name, p);
				err = 1;
				goto exit;
			}
			valstr = *argv;
			argv++;
			t->consumed++;
		}
	} else {
		t->positional = TRUE;
		valstr = p;
	}

	/* parse valstr as int just in case */
	if (valstr) {
		t->val = (int)strtol(valstr, &endptr, 0);
		if (*endptr == '\0') {
				/* not all the value string was parsed by strtol */
			t->good_int = TRUE;
		}
	}

	t->valstr = valstr;

 exit:
	if (err == 1)
		t->opt = '?';
	
	return err;
}

/* convert rate internal 500 Kbits/s units to string in Mbits/s format, like "11", "5.5" */
static char *rate_int2string(char *buf, int val)
{
	if ((val == -1) || (val == 0))
		sprintf(buf, "auto");
	else
		sprintf(buf, "%d%s Mbps", (val / 2), (val & 1) ? ".5" : "");
	return (buf);
}
